#ifndef DVS_FILE_READER_H
#define DVS_FILE_READER_H

#ifdef _WIN32
#define DVSENSE_API __declspec(dllexport)
#else
#define DVSENSE_API
#endif // _WIN32

#include <memory>
#include "DvsenseHal/camera/DvsCameraUtils.hpp"
#include "DvsenseBase/EventBase/EventTypes.hpp"
#include "DvsenseBase/Utils/TypeUtils.hpp"

namespace dvsense
{
    /** \~english @brief DVS File Reader class
     * \~chinese @brief DVS 文件读取器类
     */
    class DVSENSE_API DvsFileReader
    {
    public:
        /** \~english @brief Default constructor
         * \~chinese @brief 默认构造函数
         */
        DvsFileReader() {};

        /** \~english @brief Virtual destructor
         * \~chinese @brief 虚析构函数
         */
        virtual ~DvsFileReader() { };

        /** \~english @brief Create a File Reader object
         * \~english @param filepath Path to the file
         * \~english @return std::unique_ptr<DvsFileReader> Unique pointer to the created file reader
         * \~chinese @brief 创建一个文件读取器对象
         * \~chinese @param filepath 文件路径
         * \~chinese @return std::unique_ptr<DvsFileReader> 创建的文件读取器的唯一指针
         */
        static std::unique_ptr<DvsFileReader> createFileReader(std::string filepath);

        /** \~english @brief Loads a RAW file for event processing.
         * \~english @param filepath The full path of the RAW file to be loaded.
         * \~english @return True if the file was successfully loaded, false otherwise.
         * \~english @note This function might take significant time to complete depending on the file size.
         * \~chinese @brief 加载RAW文件以处理事件。
         * \~chinese @param filepath 要加载的RAW文件的完整路径。
         * \~chinese @return 如果文件加载成功，则返回true，否则返回false。
         * \~chinese @note 根据文件大小，此函数可能需要较长的时间来完成。
         */
        virtual bool loadFile() = 0;

        /** \~english @brief Get the Seek start of TimeStamp in the file
         * \~english @param start_timestamp Reference to the start timestamp.
         * \~english @return True if the file has timestamp information, true is returned.
         * \~english @return False if there is no timestamp information in the file, false is returned.
         * \~chinese @brief 获取文件中时间戳的起始位置。
         * \~chinese @param start_timestamp 时间戳起始位置的引用。
         * \~chinese @return 如果文件中有时间戳信息，则返回true。
         * \~chinese @return 如果文件中没有时间戳信息，则返回false。
         */
        virtual bool getStartTimeStamp(TimeStamp& start_timestamp) = 0;

        /** \~english @brief Get the Seek end of TimeStamp in the file
         * \~english @param end_timestamp Reference to the end timestamp.
         * \~english @return True if the file has timestamp information, true is returned.
         * \~english @return False if there is no timestamp information in the file, false is returned.
         * \~chinese @brief 获取文件中时间戳的结束位置。
         * \~chinese @param end_timestamp 时间戳结束位置的引用。
         * \~chinese @return 如果文件中有时间戳信息，则返回true。
         * \~chinese @return 如果文件中没有时间戳信息，则返回false。
         */
        virtual bool getEndTimeStamp(TimeStamp& end_timestamp) = 0;

        /** \~english @brief Seeks to a specific timestamp in the file.
         * \~english @param t The target timestamp to seek to.
         * \~english @return True if the seek operation was successful, false if the timestamp was not found.
         * \~chinese @brief 寻找文件中的特定时间戳。
         * \~chinese @param t 要寻找的目标时间戳。
         * \~chinese @return 如果寻址操作成功，则返回true，如果未找到时间戳，则返回false。
         */
        virtual bool seekTime(TimeStamp t) = 0;

        /** \~english @brief Gets the timestamp of the current file decode location
         * \~english @return TimeStamp 
         * \~english @note When reading a fixed number of events or at a fixed time interval,
         * the timestamp of the current file location may not correspond to the timestamp of the returned event.
         * \~chinese @brief 获取当前文件解码位置的时间戳。
         * \~chinese @return TimeStamp 
         * \~chinese @note 当以固定数量的事件或固定时间间隔读取时，
         * 当前文件位置的时间戳可能与返回的事件的时间戳不对应。
         */
        virtual TimeStamp getCurrentPosTimeStamp() = 0;

        /** \~english @brief Gets the number of events from the start of the file to the current file decoding location
         * \~english @return events num 
         * \~chinese @brief 获取从文件开头到当前文件解码位置的事件数量。
         * \~chinese @return 事件数量
         */
        virtual uint64_t getCurrentPosEventNum() = 0;

        /** \~english @brief Retrieves a specified number of events from the current file position.
         * \~english @param n The number of events to retrieve.
         * \~english @return A shared pointer to an Event2DVector containing the retrieved events.
         * \~english @warning This function is not thread-safe and the buffer will be overwritten on the next call.
         * \~chinese @brief 从当前位置检索指定数量的事件。
         * \~chinese @param n 要检索的事件数量。
         * \~chinese @return 包含检索到的事件的 Event2DVector 的共享指针。
         * \~chinese @warning 此函数不是线程安全的，并且缓冲区将在下次调用时被覆盖。
         */
        virtual std::shared_ptr<Event2DVector> getNEvents(uint64_t n) = 0;

        /** \~english @brief Retrieves a specified number of events starting from a given timestamp.
         * \~english @param start The timestamp to start reading from.
         * \~english @param n The number of events to retrieve.
         * \~english @return A shared pointer to an Event2DVector containing the retrieved events.
         * \~english @warning This function is not thread-safe and the buffer will be overwritten on the next call.
         * \~chinese @brief 从给定的时间戳开始检索指定数量的事件。
         * \~chinese @param start 开始读取的时间戳。
         * \~chinese @param n 要检索的事件数量。
         * \~chinese @return 包含检索到的事件的 Event2DVector 的共享指针。
         * \~chinese @warning 此函数不是线程安全的，并且缓冲区将在下次调用时被覆盖。
         */
        virtual std::shared_ptr<Event2DVector> getNEvents(TimeStamp start, uint64_t n) = 0;

        /** \~english @brief Retrieves events within a specific time interval from the current position.
         * \~english @param start The timestamp to start reading from.
         * \~english @param interval The time interval during which to retrieve events.
         * \~english @return A shared pointer to an Event2DVector containing the retrieved events.
         * \~english @warning This function is not thread-safe and the buffer will be overwritten on the next call.
         * \~chinese @brief 从当前位置检索特定时间间隔内的事件。
         * \~chinese @param start 开始读取的时间戳。
         * \~chinese @param interval 要检索事件的时间间隔。
         * \~chinese @return 包含检索到的事件的 Event2DVector 的共享指针。
         * \~chinese @warning 此函数不是线程安全的，并且缓冲区将在下次调用时被覆盖。
         */
        virtual std::shared_ptr<Event2DVector> getNTimeEvents(TimeStamp start, TimeStamp interval) = 0;

        /** \~english @brief Retrieves events within a specific time interval from the current position.
         * \~english @param interval The time interval during which to retrieve events.
         * \~english @return A shared pointer to an Event2DVector containing the retrieved events.
         * \~english @warning This function is not thread-safe and the buffer will be overwritten on the next call.
         * \~chinese @brief 从当前位置检索特定时间间隔内的事件。
         * \~chinese @param interval 要检索事件的时间间隔。
         * \~chinese @return 包含检索到的事件的 Event2DVector 的共享指针。
         * \~chinese @warning 此函数不是线程安全的，并且缓冲区将在下次调用时被覆盖。
         */
        virtual std::shared_ptr<std::vector<Event2D>> getNTimeEvents(TimeStamp interval) = 0;

        /**
         * \~english @brief Get the width of the sensor used to capture the events
         * \~english @return uint16_t 
         * \~chinese @brief 获取用于捕获事件的传感器的宽度
         * \~chinese @return uint16_t 
         */
        virtual uint16_t getWidth() const = 0;

        /**
         * \~english @brief Get the height of the sensor used to capture the events
         * \~english @return uint16_t 
         * \~chinese @brief 获取用于捕获事件的传感器的高度
         * \~chinese @return uint16_t 
         */
        virtual uint16_t getHeight() const = 0;
    };

    /**
     * \~english @brief unique_ptr to manage DVS file reader
     * \~chinese @brief 用于管理DVS文件读取器的unique_ptr
     */
    typedef DVSENSE_API std::unique_ptr<DvsFileReader> DvsFile;

} // namespace dvsense

#endif //DVS_FILE_READER_H